# Generated by Django 3.0.9 on 2020-08-15 16:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0020_auto_20200803_1702'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailkeyvalueauthentication',
            name='change_password_date',
            field=models.DateTimeField(blank=True, default=None, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='emailkeyvalueauthentication',
            name='change_password_token',
            field=models.CharField(blank=True, default=None, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='phonekeyvalueauthentication',
            name='change_password_date',
            field=models.DateTimeField(blank=True, default=None, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='phonekeyvalueauthentication',
            name='change_password_token',
            field=models.CharField(blank=True, default=None, max_length=255, null=True),
        ),
    ]
